/* Copyright (c) 2006 Bradley C. Kuszmaul */
#include <fftw.h>
#include "transpose-base.h"

void tran_base (val_t *A, val_t *B, long N, long A_nrows, long B_nrows) {
    long i,j;
    val_t tmp[BASE][BASE];
    /* The problem with the simpler code is that performance suffers when
     * there are associativity conflicts, which is the common case when the stride is 2^k.
     * I know of no cache-oblivious way to avoid this problem.  The best known
     * solution is to stop the recursion at Theta(sizeof(L1))-size matrices.  (On Itanium, it may be
     * size of (L2)), copy the input leaf matrix into an intermediate buffer of small stride, and copy
     * the buffer into the output leaf matrix. */
    for (j=0; j<B_nrows; j++) {
	for (i=0; i<A_nrows; i++) {
	    tmp[i][j] = A[i+j*N];
	}
    }
    for (i=0; i<A_nrows; i++) {
	for (j=0; j<B_nrows; j++) {
	    val_t Bji = B[j+i*N];
	    B[j+i*N] = tmp[i][j];
	    tmp[i][j] = Bji;
	}
    }
    for (j=0; j<B_nrows; j++) {
	for (i=0; i<A_nrows; i++) {
	    A[i+j*N] = tmp[i][j];
	}
    }
}
